/*
  dmp/device.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "dmp.h"

#include <cstdint>
#include <vector>

namespace ACN::DMP {

/**
 * @brief The ACN::DMP::Device class
 */
class Device
{
public:
  Device();
  ~Device();

  virtual void dmpReceiver(PDU::Block<DMP::Pdu>);

protected:
  virtual void dmpGet(PDU::Message<DMP::Pdu>);
  virtual void dmpSet(PDU::Message<DMP::Pdu>);

  virtual uint32_t getProperty(const unsigned int address) const;
  virtual void setProperty(const unsigned int address, const int value);

  /// \cite DMP 5.1.1 Property addresses
  /// Within a component, properties are addressed in a one dimensional
  /// property array.
  std::vector<uint32_t> * properties = nullptr;

  /// \cite DMP 5.1.2 Relative Addresses
  /// Devices shall maintain a record of the last address used
  uint32_t last_address_used = 0;

};

} // namespace ACN::DMP
