/*
  device.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "personality.h"

#include <cstdint>
#include <vector>

namespace DMX {

/**
 * @brief The Device class
 */
class Device
{
public:
  Device();
  virtual ~Device();

  uint16_t footprint() const;
  uint8_t  personality() const;
  uint8_t  personalityCount() const;
  uint16_t address() const;

  virtual void setPersonality(uint8_t mode);
  virtual bool setAddress(uint16_t start);

protected:
  std::vector<Personality*> personalities_; //!< available personalities
  uint8_t  personality_;                    //!< index of current personality
  uint16_t startAddress_;                   //!< DMX start address
};


} // namespace DMX
