/*
  dmx.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <cstdint>

namespace DMX {

  /// \cite DMX
  ///  Table D1 - Reserved START Codes
  static const uint8_t E111_NULL_START       = 0;
  static const uint8_t E111_ASC_TEXT_ASCII   = 23;
  static const uint8_t E111_ASC_TEST         = 85;
  static const uint8_t E111_ASC_TEXT_UTF8    = 144;
  static const uint8_t E111_ASC_MANUFACTURER = 145;
  static const uint8_t E111_ASC_SIP          = 207;

  /// \cite DMX
  /// 8.5.3.1 Alternate START code refresh interval
  /// A DMX512 transmitter interleaving NULL START Code packets with
  /// Alternate START Code packets shall send a NULL START Code packet at least
  /// once per second.
  static const uint16_t E111_DATA_LOSS_TIMEOUT = 1000;

} // namespace DMX
