/*
  responder.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "device.h"
#include "message.h"

#include <deque>

namespace RDM {

/**
 * @brief The Responder class
 */
class Responder
    : public Device
{
public:
  Responder(UID id, Device* parent = nullptr);
  virtual ~Responder();

  union {
    uint16_t control_field;
    struct {
      bool managed_proxy_flag   : 1;
      bool subdevice_flag       : 1;
      bool bootloader_flag      : 1;
      bool proxied_device_flag  : 1;
    };
  };

  void receive(const std::vector<uint8_t>& data);
  virtual void reset(bool hard) override;

protected:
  virtual void send(const std::vector<uint8_t>& data);
  virtual void send(MsgPtr message);
  virtual void receive(const MsgPtr message);

  virtual void rxDiscovery(const MsgPtr message, MsgPtr response);
  virtual void rxGet(const MsgPtr message, MsgPtr response);
  virtual void rxSet(const MsgPtr message, MsgPtr response);

  void actionDiscoverUniqueBranch   (const MsgPtr message, MsgPtr response);
  void actionDiscoveryMute          (const MsgPtr message, MsgPtr response);
  void actionDiscoveryUnmute        (const MsgPtr message, MsgPtr response);
  void actionGetCommsStatus         (const MsgPtr message, MsgPtr response);
  void actionSetCommsStatus         (const MsgPtr message, MsgPtr response);
  void actionGetQueuedMessage       (const MsgPtr message, MsgPtr response);
  void actionGetStatusMessages      (const MsgPtr message, MsgPtr response);
  void actionGetStatusIdDescription (const MsgPtr message, MsgPtr response);

private:
  bool     discovery_mute_flag_     = false;
  uint16_t short_message_counter_   = 0;
  uint16_t length_mismatch_counter_ = 0;
  uint16_t checksum_fail_counter_   = 0;
  MsgPtr   last_status_message_;

};


} // namespace RDM
