/*
  rlp.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <cstdint>
#include <memory>
#include "acn/pdu.h"
#include "uuid/uuid.h"

namespace ACN::RLP {

/**
 * @brief 2.6.1.2.2. Header Field in Root Layer PDUs
 *
 * The Header field in Root Layer PDUs shall contain the CID of the component
 * that generated the PDU (the Source CID).
 */
struct rlp_header : PDU::pdu_header
{
  UUID::uuid cid;   //!< Component's UUID

  size_t streamSize() const override;
  void   iStream(PDU::Stream) override;
  void   oStream(PDU::Stream) const override;
};


/**
 * @brief The RLP::Pdu class
 */
class Pdu
  : public PDU::Pdu
{
public:
  Pdu();
  void iStream(PDU::Stream) override;
};


/**
 * @brief The message_transport struct
 */
struct message_transport
    : PDU::pdu_stream_object
{
  PDU::Block<RLP::Pdu> root;  //!< root PDU block

  void iStream(PDU::Stream) override;
  void oStream(PDU::Stream) const override;
};

} // ACN::RLP
