/*
  rlp/tcp.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <cstdint>
#include "rlp.h"
#include "acn/pdu.h"

// EPI 33. ACN Root Layer Protocol Operation on TCP

namespace ACN::RLP::TCP {

/**
 * @brief 3. Frame Preamble Format
 */
struct transport
  : RLP::message_transport
{
  transport(bool filled = false);

  uint8_t  acn_id[12];        //!< 3.1 Packet Identifier
  uint32_t length;            //!< 3.2 PDU Block Size

  size_t   streamSize() const override { return sizeof(acn_id) + 4 +
                                                root.streamSize(); }
  void     iStream(PDU::Stream) override;
  void     oStream(PDU::Stream) const override;
};


/**
 * @brief 3.1 Packet Identifier
 *  The ACN Packet Identifier shall be the text string “ASC-E1.17\0\0\0”
 *  encoded in [ASCII].
 */
static const uint8_t ACN_PACKET_IDENTIFIER[] = { 0x41, 0x53, 0x43, 0x2d,
                                                 0x45, 0x31, 0x2e, 0x31,
                                                 0x37, 0x00, 0x00, 0x00 };
/**
 * @brief 3.2 PDU Block Size INDEFINITE
 */
static const uint32_t INDEFINITE = 0xffffffff;

} // ACN::RLP::TCP
