/*
  sdt/leader.cpp

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "leader.h"

namespace ACN::SDT {

/**
 * @brief Leader::Leader
 * @param cid
 */
Leader::Leader(UUID::uuid cid)
  : ACN::RLP::Component(cid)
{
  fctn_ = "OpenLCP SDT Leader";

  /**
   * 1.3.1. Recursive use of PDU Format in Other Protocols
   * ... within ACN as currently specified, there is just one higher layer
   * protocol defined which interfaces directly to the root layer:
   *    Session Data Transport
   */
  RlpRegisterVector(SDT_PROTOCOL_ID, std::bind(&Leader::SdtReceiver, this,
                                               std::placeholders::_1));
}


/**
 * @brief Leader::SdtReceiver
 * @param root
 */
void Leader::SdtReceiver(PDU::Message<RLP::Pdu> root)
{
  root->createDataBlock<SDT::Pdu>();
  auto block = std::static_pointer_cast<PDU::Block<SDT::Pdu>>(root->data());

  for(auto const &sdt : *block->pdu) {
    switch(sdt->vector()) {
      /// \todo handle SDT
      default:
        break;
    }
  }
}


/**
 * @brief Leader::SdtJoinSession
 */
void Leader::SdtJoinSession()
{

};


/**
 * @brief Leader::SdtGetSessions
 */
void Leader::SdtGetSessions()
{

};


/**
 * @brief Leader::SdtSessions
 */
void Leader::SdtSessions()
{

};

} // namespace ACN::SDT
